/**
 * @file
 *
 * @ingroup rtems_bsd_rtems
 *
 * @brief VFS to RTEMS LibIO interface.
 */

/*
 * Copyright (c) 2020 Chris Johns.  All rights reserved.
 *
 *  Contemporary Software
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <machine/rtems-bsd-kernel-space.h>

#include <sys/errno.h>
#include <sys/fcntl.h>
#include <sys/file.h>
#include <sys/filedesc.h>
#include <sys/kernel.h>
#include <sys/malloc.h>
#include <sys/namei.h>
#include <sys/time.h>
#include <sys/proc.h>
#include <sys/syscallsubr.h>
#include <sys/sysproto.h>
#include <sys/vnode.h>

#include <rtems/libio.h>
#include <rtems/seterr.h>

static void
rtems_bsd_vfs_loc_hold(
  rtems_filesystem_location_info_t *loc
)
{
  if (loc != NULL) {
    struct vnode *vp = rtems_bsd_libio_loc_to_vnode(loc);
    if (vp != NULL) {
      VI_LOCK(vp);
      vrefl(vp);
      VI_UNLOCK(vp);
    }
  }
}

static void
rtems_bsd_vfs_loc_drop(
  rtems_filesystem_location_info_t *loc
)
{
  if (loc != NULL) {
    struct vnode *vp = rtems_bsd_libio_loc_to_vnode(loc);
    if (vp != NULL) {
      vrele(vp);
    }
  }
}

void
rtems_bsd_vfs_mt_entry_lock(
  const rtems_filesystem_mount_table_entry_t *mt_entry
)
{
  struct vnode *vp =
    rtems_bsd_libio_loc_to_vnode(&mt_entry->mt_fs_root->location);
  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: lock: vn=%p\n", vp);
  vhold(vp);
}

void
rtems_bsd_vfs_mt_entry_unlock(
  const rtems_filesystem_mount_table_entry_t *mt_entry
)
{
  struct vnode *vp =
    rtems_bsd_libio_loc_to_vnode(&mt_entry->mt_fs_root->location);
  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: unlock: vn=%p\n", vp);
  vdrop(vp);
}

void
rtems_bsd_vfs_eval_path(
  rtems_filesystem_eval_path_context_t *ctx
)
{
  struct thread *td = curthread;
  struct filedesc *fdp = td->td_proc->p_fd;
  struct nameidata nd;
  struct vnode *vp;
  struct vnode *dvp;
  const char *path;
  u_long op;
  u_long flags = 0;
  int eval_flags;
  bool isdot = false;
  int error;

  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: eval_path: t=%s:%d (%s) cloc=%p\n",
           ctx->token, ctx->tokenlen, ctx->path,
           rtems_bsd_libio_loc_to_vnode(&ctx->currentloc));

  eval_flags = rtems_filesystem_eval_path_get_flags(ctx);

  flags |= (eval_flags & RTEMS_FS_FOLLOW_LINK) != 0 ? FOLLOW : NOFOLLOW;
  if ((eval_flags & RTEMS_FS_MAKE) != 0) {
    op = CREATE;
  } else {
    op = LOOKUP;
  }

  if (ctx->token != NULL) {
    vp = rootvnode;
    path = ctx->token;
  } else {
    vp = rtems_bsd_libio_loc_to_vnode(&ctx->currentloc);
    /*
     * If the root node handle:
       *   ..
       *   ../
       *   ../dev
       */
    if ((vp->v_vflag & VV_ROOT) &&
        ctx->pathlen >= 2 &&
        rtems_filesystem_is_parent_directory(ctx->path, 2)) {
      if (ctx->currentloc.mt_entry->mt_point_node == NULL) {
        rtems_filesystem_eval_path_error(ctx, EIO);
        return;
      }
      if (false && ctx->pathlen > 2) {
        rtems_filesystem_eval_path_next_token(ctx);
        rtems_filesystem_eval_path_eat_delimiter(ctx);
      }
      rtems_filesystem_eval_path_restart(ctx, &ctx->currentloc.mt_entry->mt_point_node);
      return;
    }

    if (rtems_filesystem_is_current_directory(ctx->path, ctx->pathlen)) {
      isdot = true;
    }
    path = ctx->path;
  }

  if (vp != rootvnode) {
    flags |= NOCROSSMOUNT;
    vref(rootvnode);
    vget(vp, LK_SHARED, td);
  }

  NDINIT_ATVP(&nd, op, flags, WANTPARENT | UIO_USERSPACE, path, vp, td);
  error = namei(&nd);

  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: eval_path: namei=%d:%s cwd=%p vp=%p dvp=%p mp=%p\n",
           error, strerror(error), vp, nd.ni_vp, nd.ni_dvp,
           nd.ni_vp ? nd.ni_vp->v_mountedhere : NULL);

  if (error != 0) {
    goto err;
  }

  if (vp != rootvnode) {
    VOP_UNLOCK(vp, 0);
  }

  vp = nd.ni_vp;
  dvp = nd.ni_dvp;

  if (vp->v_type == VDIR && vp->v_mountedhere != NULL) {
    struct mount *mp = vp->v_mountedhere;
    error = vfs_busy(mp, 0);
    if (error != 0) {
      goto err;
    }
    error = VFS_ROOT(mp, LK_EXCLUSIVE, &vp);
    printf("]]##> %d %p %p\n", error, mp, vp);
    vfs_unbusy(mp);
    if (error != 0) {
      goto err;
    }
  }

  rtems_bsd_libio_loc_set_vnode(&ctx->currentloc, vp);
  rtems_bsd_libio_loc_set_vnode_parent(&ctx->currentloc, dvp);
  rtems_bsd_vfs_loc_hold(&ctx->currentloc);
  NDFREE(&nd, 0);

  /*
   * Tell RTEMS we have consumed the path.
   */
  rtems_filesystem_eval_path_clear_path(ctx);

  return;

err:
  NDFREE(&nd, 0);
  rtems_filesystem_eval_path_error(ctx, error);
  return;
}

int
rtems_bsd_vfs_clonenode(
  rtems_filesystem_location_info_t *loc
)
{
  rtems_bsd_vfs_loc_hold(loc);
  if (RTEMS_BSD_VFS_TRACE) {
    struct vnode *vp = rtems_bsd_libio_loc_to_vnode(loc);
    printf("bsd: vfs: clonenode: %p vn=%p (%d/%d)\n",
           loc, vp, vp->v_holdcnt, vrefcnt(vp));
  }
  return 0;
}

void
rtems_bsd_vfs_freenode(
  const rtems_filesystem_location_info_t *loc
)
{
  if (RTEMS_BSD_VFS_TRACE) {
    struct vnode *vp = rtems_bsd_libio_loc_to_vnode(loc);
    printf("bsd: vfs:  freenode: %p vn=%p (%d/%d)\n",
           loc, vp, vp->v_holdcnt, vrefcnt(vp));
    rtems_bsd_vfs_loc_drop(RTEMS_DECONST(rtems_filesystem_location_info_t*, loc));
  }
}

int
rtems_bds_vfs_link(
  const rtems_filesystem_location_info_t *parentloc,
  const rtems_filesystem_location_info_t *targetloc,
  const char *name,
  size_t namelen
)
{
  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: link\n");
  return EOPNOTSUPP;
}

int rtems_bsd_vfs_fchmod(
  const rtems_filesystem_location_info_t *loc, mode_t mode
)
{
  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: fchmod\n");
  return EOPNOTSUPP;
}

int
rtems_bsd_vfs_chown(
  const rtems_filesystem_location_info_t *loc,
  uid_t owner,
  gid_t group
)
{
  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: chown\n");
  return EOPNOTSUPP;
}

int
rtems_bsd_vfs_mount(
  rtems_filesystem_mount_table_entry_t *mt_entry
)
{
  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: mount\n");
  return EOPNOTSUPP;
}

int
rtems_bsd_vfs_fsmount_me(
  rtems_filesystem_mount_table_entry_t *mt_entry,
  const void *data
)
{
  struct thread *td = curthread;
  struct mount_args* args = (struct mount_args*) data;
  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: fsmount_me\n");
  return sys_mount(td, args);
}

int
rtems_bsd_vfs_unmount(
  rtems_filesystem_mount_table_entry_t *mt_entry
)
{
  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: unmount\n");
  return EOPNOTSUPP;
}

void
rtems_bsd_vfs_fsunmount_me(
  rtems_filesystem_mount_table_entry_t *mt_entry
)
{
  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: fsunmount_me: %p\n",
      rtems_bsd_libio_loc_to_vnode(&mt_entry->mt_fs_root->location));
  rtems_bsd_vfs_loc_drop(&mt_entry->mt_fs_root->location);
}

bool
rtems_bsd_vfs_are_nodes_equal(
  const rtems_filesystem_location_info_t *a,
  const rtems_filesystem_location_info_t *b
)
{
  struct vnode *avp = rtems_bsd_libio_loc_to_vnode(a);
  struct vnode *bvp = rtems_bsd_libio_loc_to_vnode(b);
  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: nodes_equal: %s (%p == %p)\n",
           avp == bvp ? "equal" : "not-equal", avp, bvp);
  return avp == bvp;
}

int
rtems_bsd_vfs_mknod(
  const rtems_filesystem_location_info_t *parentloc,
  const char *name,
  size_t namelen,
  mode_t mode,
  dev_t dev
)
{
  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: mknod\n");
  return EOPNOTSUPP;
}

int
rtems_bsd_vfs_rmnod(
  const rtems_filesystem_location_info_t *parentloc,
  const rtems_filesystem_location_info_t *loc
)
{
  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: rmnod\n");
  return EOPNOTSUPP;
}

int
rtems_bsd_vfs_utime(
  const rtems_filesystem_location_info_t *loc,
  time_t actime,
  time_t modtime
)
{
  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: utime\n");
  return EOPNOTSUPP;
}

int
rtems_bsd_vfs_symlink(
  const rtems_filesystem_location_info_t *parentloc,
  const char *name,
  size_t namelen,
  const char *target
)
{
  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: symlink\n");
  return EOPNOTSUPP;
}

ssize_t
rtems_bsd_vfs_readlink(
  const rtems_filesystem_location_info_t *loc,
  char *buf,
  size_t bufsize
)
{
  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: readlink\n");
  return EOPNOTSUPP;
}

int
rtems_bsd_vfs_rename(
  const rtems_filesystem_location_info_t *oldparentloc,
  const rtems_filesystem_location_info_t *oldloc,
  const rtems_filesystem_location_info_t *newparentloc,
  const char *name,
  size_t namelen
)
{
  struct thread *td = curthread;
  char* from = NULL;
  char* to = NULL;
  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: rename\n");
  return kern_renameat(td, AT_FDCWD, from, AT_FDCWD, to, UIO_USERSPACE);
}

int
rtems_bsd_vfs_statvfs(
  const rtems_filesystem_location_info_t *loc,
  struct statvfs *buf
)
{
  if (RTEMS_BSD_VFS_TRACE)
    printf("bsd: vfs: statvfs\n");
  return EOPNOTSUPP;
}
