/*
 * Copyright (c) 2016 Chris Johns <chrisj@rtems.org>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#define RTEMS_TINYSSHD_TRACE (1)

#include <errno.h>
#include <stdarg.h>
#include <stdlib.h>
#include <unistd.h>

#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <rtems/tinysshd.h>

extern int tinyssh_main(int argc, char **argv, int in, int out);
extern int tinyssh_printkey(int argc, char **argv);
extern int tinyssh_makekey(int argc, char **argv);

/**
 * Destroy times.
 */
#define RTEMS_TINYSSHD_POLL_WAIT    (10000)
#define RTEMS_TINYSSHD_TIMEOUT_STOP (5)

typedef struct
{
  rtems_id session_id;
  int      session_fd;
  FILE*    pty_std[3];
} tinysshd_session_data;

/**
 * Tinyssh runtime data.
 */
typedef struct
{
  rtems_tinysshd_config config;
  rtems_id              lock;
  rtems_id              server_task;
  volatile bool         server_running;
  volatile bool         server_finished;
  int                   listener;
  int                   connections;
} rtems_tinysshd;

static rtems_tinysshd* sshd;

void
rtems_tinysshd_printf(const char* format, ...)
{
#if RTEMS_TINYSSHD_TRACE
  va_list ap;
  va_start(ap, format);
  vprintf(format, ap);
  va_end(ap);
#endif
}

static rtems_status_code
rtems_tinysshd_lock(void)
{
  if (sshd->lock != 0) {
    rtems_status_code sc;
    sc = rtems_semaphore_obtain(sshd->lock, RTEMS_WAIT, RTEMS_NO_TIMEOUT);
    if (sc != RTEMS_SUCCESSFUL) {
      fprintf("error: tinysshd: lock: %s\n", rtems_status_text(sc));
      return sc;
    }
  }
  return RTEMS_SUCCESSFUL;
}

static rtems_status_code
rtems_tinysshd_unlock(void)
{
  if (sshd->lock != 0) {
    rtems_status_code sc;
    sc = rtems_semaphore_release(sshd->lock);
    if (sc != RTEMS_SUCCESSFUL) {
      fprintf(stderr, "error: tinysshd: unlock: %s\n", rtems_status_text(sc));
      return sc;
    }
  }
  return RTEMS_SUCCESSFUL;
}

static rtems_status_code
rtems_tinysshd_lock_create(void)
{
  #define LOCK_ATTRIBUTES \
    RTEMS_PRIORITY | RTEMS_INHERIT_PRIORITY | RTEMS_BINARY_SEMAPHORE
  rtems_status_code sc;
  sc = rtems_semaphore_create(rtems_build_name('S', 'S', 'H', 'D'),
                              1,
                              LOCK_ATTRIBUTES,
                              0,
                              &sshd->lock);
  if (sc != RTEMS_SUCCESSFUL) {
    fprintf(stderr, "error: tinysshd: sema create: %s\n", rtems_status_text(sc));
    return sc;
  }
  return RTEMS_SUCCESSFUL;
}

static rtems_status_code
rtems_tinysshd_lock_destroy(void)
{
  rtems_tinysshd_lock();
  if (sshd->lock != 0) {
    rtems_status_code sc;
    rtems_semaphore_release(sshd->lock);
    sc = rtems_semaphore_delete(sshd->lock);
    sshd->lock = 0;
    if (sc != RTEMS_SUCCESSFUL) {
      fprintf(stderr, "error: tinysshd: sema delete: %s\n",
	      rtems_status_text(sc));
      return sc;
    }
  }
  return RTEMS_SUCCESSFUL;
}

static rtems_status_code
rtems_tinysshd_task_create(const char*         name,
			   rtems_task_priority priority,
			   size_t              stack_size,
			   rtems_task_entry    entry_point,
			   rtems_task_argument argument,
			   rtems_id*           id)
{
  rtems_name        tname;
  rtems_status_code sc;

  tname = rtems_build_name(name[0], name[1], name[2], name[3]);

  sc = rtems_task_create (tname,
                          priority,
                          stack_size,
                          RTEMS_FLOATING_POINT | RTEMS_LOCAL,
                          RTEMS_PREEMPT | RTEMS_NO_ASR,
                          id);
  if (sc != RTEMS_SUCCESSFUL) {
    *id = 0;
    fprintf(stderr, "error: tinysshd: thread create: %s: %s\n",
	    name, rtems_status_text(sc));
    return sc;
  }

  sc = rtems_task_start(*id, entry_point, argument);
  if (sc != RTEMS_SUCCESSFUL) {
    fprintf(stderr, "error: tinyssh: thread start: %s: %s\n",
	    name, rtems_status_text(sc));
    rtems_task_delete(*id);
    *id = 0;
    return sc;
  }

  return RTEMS_SUCCESSFUL;
}

static rtems_status_code
rtems_tinysshd_task_destroy(const char*    name,
			    rtems_id       id,
			    volatile bool* finished,
			    int            timeout)
{
  while (timeout) {
    bool has_finished;
    rtems_debugger_lock();
    has_finished = *finished;
    rtems_debugger_unlock();

    if (has_finished)
      break;

    usleep(RTEMS_TINYSSHD_POLL_WAIT);
    if (timeout < RTEMS_TINYSSHD_POLL_WAIT)
      timeout = 0;
    else
      timeout -= RTEMS_TINYSSHD_POLL_WAIT;
  }

  if (timeout == 0) {
    fprintf(stderr, "tinyssh: %s not stopping, killing\n", name);
    rtems_task_delete(id);
  }

  return RTEMS_SUCCESSFUL;
}

static bool
rtems_tinysshd_server_running(void)
{
  bool running;
  rtems_tinysshd_lock();
  running = sshd->server_running;
  rtems_tinysshd_unlock();
  return running;
}

static void
rtems_tinysshd_listener_close(void)
{
  if (sshd->listener >= 0) {
    close(sshd->listener);
    sshd->listener = -1;
  }
}

static bool
rtems_tinysshd_listener_open(void)
{
  struct sockaddr_in addr;
  socklen_t          opt;
  socklen_t          len;
  bool               running;
  struct timeval     timeout;
  int                r;

  sshd->listener = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
  if (sshd->listener < 0) {
    fprintf(stderr, "error: tinysshd: tcp remote: socket: (%d) %s\n",
	    errno, strerror(errno));
    return false;
  }

  opt = 1;
  r = setsockopt(sshd->listener,
                 SOL_SOCKET,
                 SO_REUSEADDR,
                 (char *) &opt,
                 sizeof(opt));
  if (r < 0) {
    rtems_tinysshd_listener_close();
    fprintf(stderr, "error: tinysshd: tcp remote: setsocket: reuseaddr: (%d) %s\n",
	    errno, strerror(errno));
    return false;
  }

  opt = 1;
  r = setsockopt(sshd->listener,
                 SOL_SOCKET,
                 SO_KEEPALIVE,
                 (char *) &opt,
                 sizeof(opt));
  if (r < 0) {
    rtems_tinysshd_listener_close();
    fprintf(stderr, "error: tinysshd: tcp remote: setsocket: keepalive: (%d) %s\n",
	    errno, strerror(errno));
    return false;
  }

  memset(&addr, 0, sizeof(addr));
  addr.sin_family = PF_INET;
  addr.sin_port = htons(sshd->config.port);
  addr.sin_addr.s_addr = INADDR_ANY;

  r = bind(sshd->listener, (struct sockaddr *) &addr, sizeof(addr));
  if (r < 0) {
    rtems_tinysshd_listener_close();
    fprintf(stderr, "error: tinysshd: tcp remote: bind: (%d) %s\n",
	    errno, strerror(errno));
    return false;
  }

  r = listen(sshd->listener, sshd->config.backlog);
  if (r < 0) {
    rtems_tinysshd_listener_close();
    fprintf(stderr, "error: tinysshd: tcp remote: listen: (%d) %s\n",
	    errno, strerror(errno));
    return false;
  }

  return true;
}

static int
rtems_tinysshd_connect(void)
{
  int fd = - 1;

  while (true) {
    int                ld;
    struct sockaddr_in addr;
    socklen_t          opt;
    socklen_t          len;
    bool               running;
    int                r;

    ld = sshd->listener;
    len = sizeof(addr);

    rtems_tinysshd_unlock();

    fd = accept(ld, (struct sockaddr *) &addr, &len);

    rtems_tinysshd_lock();

    if (fd < 0) {
      /*
       * EBADF means the listen socket has been closed, no error printed.
       */
      if (errno != EBADF)
	fprintf(stderr, "error: tinysshd: accept: (%d) %s\n",
		errno, strerror(errno));
      return -1;
    }

    if (!sshd->server_running) {
      close(fd);
      errno = EIO;
      return -1;
    }

    opt = 1;
    r = setsockopt(fd,
		   SOL_SOCKET, SO_KEEPALIVE,
		   (char*) &opt,
		   sizeof(opt));
    if (r < 0) {
      int errno_ = errno;
      close(fd);
      fprintf(stderr, "error: tinysshd: connect: set keepalive: (%d) %s\n",
	      errno, strerror(errno));
      errno = errno_;
      return -1;
    }

    opt = 1;
    r = setsockopt(fd,
		   IPPROTO_TCP, TCP_NODELAY,
		   (char*) &opt, sizeof(opt));
    if (r < 0) {
      int errno_ = errno;
      close(fd);
      fprintf(stderr, "error: tinysshd: connect: set no-delay: (%d) %s\n",
	      errno, strerror(errno));
      errno = errno_;
      return -1;
    }

    if (sshd->connections++ < sshd->config.connections) {
      fprintf(stderr, "tinnyssh: connect: %s\n", inet_ntoa(addr.sin_addr));
      break;
    }

    fprintf(stderr, "tinnyssh: connect: too many connections\n");
  }

  return fd;
}

#if 0
  ssh_errno = errno;
  bool                   logged_in = true;
  if (ssh_r == 0) {
    if (sshd->config.login_check != 0) {
      logged_in = rtems_shell_login_prompt(stdin,
					   stderr,
					   pty,
					   sshd->config.login_check);
    }

    if (logged_in)
      sshd->config.command(pty, sshd->config.arg);
  }
  else {
    if (!logged_in)
      fprintf(stderr, "error: tinysshd: login failed\n");
  }
#endif

/*
 * This is taken from telnetd. These should be merged at some point.
 */
static void
rtems_tinysshd_session(rtems_task_argument arg)
{
  tinysshd_session_data* session = (tinysshd_session_data*) arg;
  FILE*                  ostd[3] = { stdin, stdout, stderr };
  rtems_status_code      sc;
  int                    ssh_errno;
  int                    r;
  char*                  args[] = {
    "tinysshd", "-v", "-v", "/etc/sshd/keys", NULL
  };

  rtems_tinysshd_printf("sshd: session: start\n");

  sc = rtems_libio_set_private_env();

  fileno(stdout);

  if (sc != RTEMS_SUCCESSFUL) {
    fprintf(stderr, "error: tinysshd: setting private env: %s\n",
	    rtems_status_text(sc));
    rtems_task_delete(RTEMS_SELF);
  }

#if 0
  stdin  = session->pty_std[0];
  stdout = session->pty_std[1];
  stderr = session->pty_std[2];
#endif

  r = tinyssh_main(4, args, session->session_fd, session->session_fd);

  if (r < 0)
    ssh_errno = errno;

#if 0
  stdin  = ostd[0];
  stdout = ostd[1];
  stderr = ostd[2];

  fclose(session->pty_std[0]);
  fclose(session->pty_std[1]);
  fclose(session->pty_std[2]);
#endif

  if (r < 0)
    fprintf(stderr, "error: tinysshd: ssh: %s\n", strerror(ssh_errno));

  rtems_tinysshd_printf("sshd: session: end\n");

  rtems_task_delete(RTEMS_SELF);
}

static void
rtems_tinysshd_session_create(int fd)
{
  char*                  pty;
  tinysshd_session_data* session;
  rtems_status_code      sc;
  int                    i;

  session = calloc(1, sizeof(tinysshd_session_data));
  if (session == NULL) {
    fprintf(stderr, "error: tinysshd: no memory for session data\n");
    close(fd);
    return;
  }

  session->session_fd = fd;

#if 0
  /*
   * The PTY interface is broken in many ways, eg there is no lock on the table
   * search. Just use the returned pointer until it is fixed.
   */
  pty = telnet_get_pty(fd);
  if (pty == NULL) {
    fprintf(stderr, "error: tinysshd: unable to obtain PTY\n");
    free(session);
    close(fd);
    return;
  }

  for (i = 0; i < 3; i++) {
    session->pty_std[i] = fopen(pty, "r+");
    if (session->pty_std[i] == NULL) {
      int j;
      fprintf(stderr, "error: tinysshd: stdio set up: %s\n",
	      strerror(errno));
      for (j = 0; j < i; ++j)
	close(session->pty_std[j]);
      free(session);
      close(fd);
      return;
    }
  }
#endif

  sc = rtems_tinysshd_task_create("SSHs",
				  sshd->config.priority,
				  sshd->config.stack_size,
				  rtems_tinysshd_session,
				  session,
				  &session->session_id);
  if (sc != RTEMS_SUCCESSFUL) {
    for (i = 0; i < 3; ++i)
      close(session->pty_std[i]);
    free(session);
    close(fd);
  }
}

static void
rtems_tinysshd_main(rtems_task_argument arg)
{
  rtems_tinysshd_lock();

  rtems_tinysshd_printf("sshd: server: start\n");

  if (rtems_tinysshd_listener_open()) {
    while (sshd->server_running) {
      int fd = rtems_tinysshd_connect();
      if (fd < 0)
	break;
      rtems_tinysshd_session_create(fd);
    }
  }

  sshd->server_running = false;
  sshd->server_finished = true;
  rtems_tinysshd_unlock();

  rtems_tinysshd_printf("sshd: server: end\n");

  rtems_task_delete(RTEMS_SELF);
}

rtems_status_code
rtems_tinysshd_start(const rtems_tinysshd_config* config)
{
  rtems_status_code sc;

  if (config == NULL) {
    fprintf(stderr, "sshd: no config\n");
    return RTEMS_INVALID_ADDRESS;
  }

  if (config->priority == 0) {
    fprintf(stderr, "sshd: no priority set\n");
    return RTEMS_INVALID_NUMBER;
  }

  if (config->stack_size == 0) {
    fprintf(stderr, "sshd: no stack size set\n");
    return RTEMS_INVALID_NUMBER;
  }

  if (sshd != NULL) {
    fprintf(stderr, "sshd: already in use\n");
    return RTEMS_RESOURCE_IN_USE;
  }

  if (!telnet_pty_initialize()) {
    fprintf(stderr, "sshd: cannot initialize PTY driver\n");
    return RTEMS_IO_ERROR;
  }

  sshd = malloc(sizeof(rtems_tinysshd));
  if (sshd == NULL) {
    fprintf(stderr, "sshd: no memory\n");
    return RTEMS_NO_MEMORY;
  }

  sshd->config = *config;

  if (sshd->config.connections == 0)
    sshd->config.connections = 4;
  if (sshd->config.port == 0)
    sshd->config.port = 22;
  if (sshd->config.backlog == 0)
    sshd->config.backlog = 1;

  sc = rtems_tinysshd_lock_create();
  if (sc != RTEMS_SUCCESSFUL) {
    free(sshd);
    sshd = NULL;
    return sc;
  }

  sshd->server_running = true;
  sshd->server_finished = false;

  sc = rtems_tinysshd_task_create("SSHD",
				  sshd->config.priority,
				  RTEMS_MINIMUM_STACK_SIZE,
				  rtems_tinysshd_main,
				  NULL,
				  &sshd->server_task);
  if (sc != RTEMS_SUCCESSFUL) {
    rtems_tinysshd_lock_destroy();
    free(sshd);
    sshd = NULL;
    return sc;
  }

  return RTEMS_SUCCESSFUL;
}

rtems_status_code rtems_tinysshd_stop(void)
{
  rtems_status_code sc = RTEMS_SUCCESSFUL;

  if (sshd != NULL) {
    rtems_status_code sc_sc;

    rtems_tinysshd_lock();

    sshd->server_running = false;

    rtems_tinysshd_listener_close();

    rtems_tinysshd_unlock();

    sc = rtems_debugger_task_destroy("SSHD",
				     sshd->server_task,
				     &sshd->server_finished,
				     RTEMS_TINYSSHD_TIMEOUT_STOP);

    sc_sc = rtems_debugger_lock_destroy();
    if (sc_sc != RTEMS_SUCCESSFUL && sc == RTEMS_SUCCESSFUL)
      sc = sc_sc;

    free(sshd);
    sshd = NULL;
  }

  return sc;
}


rtems_status_code
rtems_tinyssh_printkey(const rtems_tinysshd_config* config)
{
  char* args[] = {
    "tinysshd-printkey", "/etc/sshd/keys", NULL
  };
  rtems_status_code sc;
  int               r;

  if (config == NULL) {
    fprintf(stderr, "sshd: no config\n");
    return RTEMS_INVALID_ADDRESS;
  }

  r = tinyssh_printkey(3, args);

  return r == 0 ? RTEMS_SUCCESSFUL : RTEMS_INVALID_NUMBER;
}
