/**
 * @file
 *
 * @brief It mounts a network file system (nfs).
 */

/*
 * Copyright (c) 2016 embedded brains GmbH.  All rights reserved.
 *
 *  embedded brains GmbH
 *  Dornierstr. 4
 *  82178 Puchheim
 *  Germany
 *  <rtems@embedded-brains.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <assert.h>
#include <stdio.h>
#include <unistd.h>

#include <fcntl.h>
#include <sys/stat.h>

#include <rtems.h>
#include <rtems/libio.h>

#include <rtems/shell.h>
#include <rtems/telnetd.h>

#include <rtems/bsd/test/network-config.h>

#define TEST_NAME "LIBBSD NFS 2"
#define TEST_STATE_USER_INPUT 1

static const mode_t mode = S_IRWXU | S_IRWXG | S_IRWXO;

volatile char gen_string[256];

static void
telnet_shell(char *name, void *arg)
{
	rtems_shell_env_t env;

	rtems_shell_dup_current_env(&env);

	env.devname = name;
	env.taskname = "TLNT";
	env.login_check = NULL;
	env.forever = false;

	rtems_shell_main_loop(&env);
}

rtems_telnetd_config_table rtems_telnetd_config = {
	.command = telnet_shell,
	.arg = NULL,
	.priority = 0,
	.stack_size = 0,
	.login_check = NULL,
	.keep_stdio = false
};

static void
test_mkdir(const char* base)
{
  assert(mkdir(base, mode) == 0);
  assert(chdir(base) == 0);
}

static void
test_open(const char* name)
{
  int fd = open(name, O_RDONLY);
  assert(fd >= 0);
  close(fd);
}

static void
test_read(const char* name)
{
  int fd = open(name, O_RDONLY);
  char buf[1024];
  ssize_t r;
  assert(fd >= 0);
  memset(buf, 0, sizeof(buf));
  r = read(fd, buf, sizeof(buf));
  assert(r > 0);
  printf("read %d\n", r);
  printf("%s", buf);
  close(fd);
}

static void
test_chdir_opendot(const char* name)
{
  struct stat sb;
  int fd;
  assert(chdir(name) == 0);
  fd = open(".", O_RDONLY);
  assert(fd >= 0);
  assert(fstat(fd, &sb) == 0);
  printf("dir: %s:\n", name);
  printf(" size: %llu\n", sb.st_size);
  printf("  uid: %i\n", sb.st_uid);
  printf("  gid: %i\n", sb.st_gid);
  printf("   mt: %l\n", sb.st_mtime);
  close(fd);
}

static void
test_main(void)
{
  static const char remote_target[] =
    "1000.100@" NET_CFG_PEER_IP ":/opt/src";
  const char* dname = "/nfs";
  const char* fname = "/nfs/dot-gitconfig";
  int rv;

  rtems_status_code sc = rtems_telnetd_initialize();
  assert(sc == RTEMS_SUCCESSFUL);

  memset(gen_string, 0, sizeof(gen_string));

  printf("Mounting NFS ...\n");

  do {
    sleep(1);

    rv = mount_and_make_target_path(&remote_target[0], "/nfs",
				    RTEMS_FILESYSTEM_TYPE_NFS, RTEMS_FILESYSTEM_READ_WRITE,
				    NULL);
    printf("mounting result: %d\n", rv);

  } while (rv != 0);

  test_open(fname);
  test_read(fname);
  test_chdir_opendot(dname);

  rtems_task_delete(RTEMS_SELF);
  assert(0);
}

#define CONFIGURE_SHELL_COMMANDS_ALL

#define DEFAULT_NETWORK_SHELL

#define CONFIGURE_MAXIMUM_DRIVERS 32

#define CONFIGURE_FILESYSTEM_NFS

#include <rtems/bsd/test/default-network-init.h>
